// If OpenGL ES
#ifdef GL_ES
    precision highp float;
#endif

// Textures
//uniform samplerCube sCubemapTexture;
//varying vec3 vCubeTexCoords;

uniform sampler2D sTexture[1];
// Second Texture
//uniform sampler2D sOtherTexture;
varying vec2 vTexCoords;

// Lighting
uniform vec4 uAmbientLight;
uniform vec4 uDiffuseLight[5];
uniform vec4 uSpecularLight[5];
uniform vec3 uLightPosition[5];
uniform float uShininess[5];

uniform int uLightCount;
uniform float uCircleInnerRadius;
uniform float uCircleOuterRadius;

varying vec3 vNormal, vPosition;
varying vec2 vPixelVertex;
varying float vLighting, vTexture;

void main(void)
{    
    if (uCircleOuterRadius > 0.0)
    {
        float dist = distance(vPixelVertex, vec2(0.0, 0.0));
        
        if (dist > (uCircleOuterRadius + uCircleInnerRadius) || dist < uCircleInnerRadius)
        {
            discard;
        }
    }

    vec4 fragmentColor = vec4(0.0);
        
    if (vTexture == 0.0 && vLighting == 0.0)
    {
        fragmentColor += uAmbientLight;
    }
    else
    {
        if (vTexture == 1.0)
        {
            fragmentColor += texture2D(sTexture[0], vTexCoords);
            // Second Texture
            //fragmentColor += texture2D(sTexture[1], vTexCoords);
            
            // This works great (sometimes) for removing the fuzzy edges of a font image
//            if (fragmentColor.a < 0.1)
//            {
//                discard;
//            }
            
            fragmentColor *= uAmbientLight;
        }
//        else if (vTexture == 2.0)
//        {
//            fragmentColor += textureCube(sCubemapTexture, vCubeTexCoords);
//            fragmentColor *= uAmbientLight;
//        } 

        if (vLighting == 1.0)
        {
            if (vTexture == 0.0)
            {
                fragmentColor += uAmbientLight;
            }
            
            for (int i = 0; i < uLightCount; i++)
            {
                vec3 N = normalize(vNormal);
                vec3 V = normalize(-vPosition);
                vec3 L = normalize(uLightPosition[i] - vPosition);
                vec3 R = reflect(-L, N);
                
                float clampShine = clamp(dot(N,L), 0.0, 1.0);
                
                fragmentColor += uDiffuseLight[i] * clampShine;
                
                if (uShininess[i] > 0.0)
                {
                    float specular = pow(max(dot(R, V), 0.0), uShininess[i]);
                
                    fragmentColor += uSpecularLight[i] * specular * clampShine;
                }
            }
        }
    }
    
    gl_FragColor = fragmentColor;
}
